@@ -67,6 +67,7 @@ CryptController::CryptController (Mail *mail, bool encrypt, bool sign,
     m_encrypt (encrypt),
     m_sign (sign),
     m_crypto_success (false),
+    m_no_smime_shown (false),
     m_proto (proto)
 {
   TSTART;
@@ -365,9 +366,13 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
 
   if (m_sign && sigFprs.empty())
     {
-      log_error ("%s:%s: Sign requested but no signing fingerprint - sending unsigned",
+      log_dbg ("%s:%s: Sign requested but no signing fingerprint - sending unsigned",
                  SRCNAME, __func__);
       m_sign = false;
+      if (!m_encrypt)
+        {
+          TRETURN 0;
+        }
     }
   if (m_encrypt && !recpFprs.size())
     {
@@ -499,7 +504,7 @@ CryptController::get_resolved_protocol () const
 /* Check that the crypt operation is resolved. This
    supports combined S/MIME and OpenPGP operations. */
 bool
-CryptController::is_resolved () const
+CryptController::is_resolved ()
 {
   /* Check that we have signing keys if necessary and at
      least one encryption key for each recipient. */
@@ -519,6 +524,60 @@ CryptController::is_resolved () const
   log_dbg ("Has OpenPGP Sig Key: %i SMIME: %i",
            hasOpenPGPSignKey, hasSMIMESignKey);
 
+  if (m_no_smime_shown)
+    {
+      /* We have already shown the message that S/MIME signing is not resolvable
+       * in this setting. Since we then talk about the following dialog we must
+       * block all other protocol resolution tries to show the dialog. */
+      return false;
+    }
+
+  if (!m_no_smime_shown && !hasSMIMESignKey && m_sign && opt.sign_default &&
+      opt.enable_smime && opt.prefer_smime)
+    {
+      if (!KeyCache::instance ()->isPopulated())
+        {
+          log_dbg ("Keycache is not loaded. Showing just the UI resolver.");
+        }
+      else if (opt.smimeNoCertSigErr)
+        {
+          /* If it comes from opt it comes from the windows registry which we
+             access with native encoding. */
+          char *msg = native_to_utf8 (opt.smimeNoCertSigErr);
+          if (!msg)
+            {
+              log_err ("Invalid setting in opt.smimeNoCertSigErr '%s'",
+                       opt.smimeNoCertSigErr);
+            }
+          else
+            {
+              gpgol_message_box (m_mail->getWindow (), msg,
+                                 _("GpgOL"), MB_OK);
+              m_no_smime_shown = true;
+              xfree (msg);
+              return false;
+            }
+        }
+      else
+        {
+          const char * msg = utf8_gettext ("No S/MIME (X509) signing certificate found.\n\n"
+                  "Your organization has configured GpgOL to sign outgoing\n"
+                  "mails with S/MIME certificates but there is no S/MIME\n"
+                  "certificate configured for your mail address.\n\n"
+                  "Please ask your Administrators for assistance or switch\n"
+                  "to OpenPGP in the next dialog.");
+          gpgol_message_box (m_mail->getWindow (), msg,
+                             _("GpgOL"), MB_OK);
+          m_no_smime_shown = true;
+          return false;
+        }
+    }
+
+  if (m_sign && !hasOpenPGPSignKey && !hasSMIMESignKey)
+    {
+      return false;
+    }
+
   if (m_encrypt)
     {
       Recipient::dump (m_recipients);
@@ -900,7 +959,7 @@ CryptController::resolve_keys ()
 }
 
 int
-CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
+CryptController::prepare_crypto()
 {
   TSTART;
   log_debug ("%s:%s",
@@ -963,6 +1022,11 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
       // Cancel
       TRETURN -2;
     }
+  if (!m_encrypt && !m_sign)
+    {
+      log_dbg ("Nothing left after resolution. Passing unencrypted.");
+      TRETURN -4;
+    }
 
   if (!m_mail->copyParent ())
     {
@@ -980,7 +1044,18 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
           TRETURN -3;
         }
     }
+  TRETURN 0;
+}
 
+int
+CryptController::do_crypto (GpgME::Error &err, std::string &r_diag, bool force)
+{
+  TSTART;
+  if (m_signer_keys.empty () && m_enc_keys.empty ()) {
+    log_err ("Do crypto called without prepared keys. Call prepare_crypto "
+             "first.");
+    TRETURN -1;
+  }
   bool do_inline = m_mail->getDoPGPInline ();
 
   if (m_proto == GpgME::CMS && do_inline)
@@ -1017,13 +1092,38 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
   ctx->setTextMode (m_proto == GpgME::OpenPGP);
   ctx->setArmor (m_proto == GpgME::OpenPGP);
 
+  /* Prepare to offer a "Forced encryption for S/MIME errors or drafts */
+  GpgME::Context::EncryptionFlags flags = GpgME::Context::EncryptionFlags::None;
+  int errVal = -1;
+  bool draft = m_mail->isDraftEncrypt ();
+  /* For openPGP or when force is used we want to use Always Trust */
+  if (m_proto == GpgME::OpenPGP || force || draft) {
+      /* Force is currently only used for S/MIME but just to make
+         this clear we add the check here */
+      if ((force || draft) && m_proto == GpgME::CMS)
+        {
+          log_dbg ("Using AlwaysTrust %s option", (draft ? "draft" : "force"));
+          /* When force is used we don't need any online verification. */
+          ctx->setOffline(true);
+          /* Rewind the input and start with a fresh output */
+          log_dbg ("Rewinding input");
+          m_input.rewind ();
+          m_output = GpgME::Data ();
+        }
+      flags = GpgME::Context::AlwaysTrust;
+  } else if (m_proto == GpgME::CMS && !force) {
+      /* This should indicate to the caller that a repeated call with
+         force might work */
+      errVal = -3;
+  }
+
   if (m_encrypt && m_sign && do_inline)
     {
       // Sign encrypt combined
       const auto result_pair = ctx->signAndEncrypt (m_enc_keys,
                                                     do_inline ? m_bodyInput : m_input,
                                                     m_output,
-                                                    GpgME::Context::AlwaysTrust);
+                                                    flags);
       const auto err1 = result_pair.first.error();
       const auto err2 = result_pair.second.error();
 
@@ -1040,13 +1140,13 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
             {
               r_diag = log.toString();
             }
-          TRETURN -1;
+          TRETURN errVal;
         }
 
       if (err1.isCanceled() || err2.isCanceled())
         {
           err = err1.isCanceled() ? err1 : err2;
-          log_debug ("%s:%s: User cancled",
+          log_debug ("%s:%s: User canceled",
                      SRCNAME, __func__);
           TRETURN -2;
         }
@@ -1068,7 +1168,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
             {
               r_diag = log.toString();
             }
-          TRETURN -1;
+          TRETURN errVal;
         }
       if (err.isCanceled())
         {
@@ -1098,13 +1198,13 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
           TRETURN -1;
         }
 
-      // Now we have the multipart throw away the rest.
+      // Now we have the multipart we do not need the output anymore.
       m_output = GpgME::Data ();
-      m_input = GpgME::Data ();
+      m_output.setEncoding(GpgME::Data::MimeEncoding);
       multipart.seek (0, SEEK_SET);
       const auto encResult = ctx->encrypt (m_enc_keys, multipart,
                                            m_output,
-                                           GpgME::Context::AlwaysTrust);
+                                           flags);
       err = encResult.error();
       if (err)
         {
@@ -1117,11 +1217,11 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
             {
               r_diag = log.toString();
             }
-          TRETURN -1;
+          TRETURN errVal;
         }
       if (err.isCanceled())
         {
-          log_debug ("%s:%s: User cancled",
+          log_debug ("%s:%s: User canceled",
                      SRCNAME, __func__);
           TRETURN -2;
         }
@@ -1129,9 +1229,10 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
     }
   else if (m_encrypt)
     {
+      m_output.setEncoding(GpgME::Data::MimeEncoding);
       const auto result = ctx->encrypt (m_enc_keys, do_inline ? m_bodyInput : m_input,
                                         m_output,
-                                        GpgME::Context::AlwaysTrust);
+                                        flags);
       err = result.error();
       if (err)
         {
@@ -1144,7 +1245,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
             {
               r_diag = log.toString();
             }
-          TRETURN -1;
+          TRETURN errVal;
         }
       if (err.isCanceled())
         {
@@ -1174,7 +1275,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
         }
       if (err.isCanceled())
         {
-          log_debug ("%s:%s: User cancled",
+          log_debug ("%s:%s: User canceled",
                      SRCNAME, __func__);
           TRETURN -2;
         }
@@ -1187,8 +1288,7 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
                  SRCNAME, __func__);
     }
 
-
-  log_debug ("%s:%s: Crypto done sucessfuly.",
+  log_debug ("%s:%s: Crypto done successfully.",
              SRCNAME, __func__);
   m_crypto_success = true;
 
@@ -1587,3 +1687,9 @@ CryptController::start_crypto_overlay ()
   m_overlay = std::unique_ptr<Overlay> (new Overlay (wid, text));
   TRETURN;
 }
+
+void
+CryptController::stop_crypto_overlay ()
+{
+  m_overlay = nullptr;
+}
